/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzMath_h

#ifndef ZzMath_h
#include "ZzMath.h"
#endif //ZzMath_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#include <cmath>

// clips a number between two clamp values
template<class T, class U, class V> T ZzClip(T x, U lo, V hi)
{
	if( x < lo ) return lo;
	if( x > hi ) return hi;
	return x;
}

// double : from -1.0 to 1.0
// 16-bit PCM max : 32767 (0x7FFF) min : -32768 (0x8000)	0 
inline int16 DoubleToINT16(double d)
{
	// $$$ very inefficient routine... we don't want overflow bugs *now*
	
	// -1 needs to be mapped to -32768
	// 1 needs to be mapped to 32767
	// 0 needs to be mapped to 0
	// IMPOSSIBLE TO DO THIS WITH LINEAR FUNCTION
	// thus, we choose to map -1 to -32767...
	// $$$ this could cause problems in round trip conversion
	// caused by a LOT of feedback... but, then, precision issues
	// in pure floating point computations is also problematic...
	const double kDoubleToInt = 32767.0;
	d = ZzClip(d,-1.0,1.0);
	int32 val = int32(d*kDoubleToInt);
	val = ZzClip(val,-32767,32767);
	return (int16)val;
}


// this function returns the negative-remainder integer division
// q of a and b which is such as (a = b*q - r) where r is
// between 0 and q-1

// very useful under some circumstances : for example, in the audio
// clip, we want to build a waveform bitmap that has a pixel for four samples
// to compute the number of pixels being given the number of frames, we use
// the nrdiv. If we have 400 frames we want 100 pixels, but if we have 
// 401 frames, we want 101 pixels

// Nr means negative remainder
template <class T> inline T ZzNrDiv( T top, T bottom )
{
	return (top-1)/bottom + 1;
}

inline float ZzFabs(float a)
{
	if(a>=0)
		return a;
	else
		return -a;
}

/*inline int32 ZzNrMod( int32 top, int32 bottom )
{
	return (top-1)/bottom + 1;
}*/

// this gives the smallest power of two greater or equal to the number supplied
// it returns the logarithm, not the actual number
inline UInt32 ZzMathLog2Cap(UInt32 val)
{
	for(UInt32 i = 0; i < 31; i++)
	{
		if( (1UL << i) >= val)
			return i;
	}
	return 32;
}
